/*
 * ATXMega32_Saintsmart_74HC595.c
 *
 * Created: 23.06.2015 08:49:11
 *  Author: Dieter
 */ 


#include "Common.h"




void Clock_init();
void Int_init();
uint8_t print_disk_info();
uint8_t sd_buffer[512];

void SSD1306_Graphic_fun(void);
void SSD1306_Graphic_fun_init(void);



int main(void)
{
	offset_t block_add = 10;
	uint16_t read_len  = 512;


	Clock_init();

	SSD1306_init();

	SSD1306_Graphic_fun_init();
	
	spiopen();

	PORTB.DIRSET = PIN3_bm;
	PORTB.OUTSET = PIN3_bm;
	
	SSD1289_init();

	if (sd_raw_init() == 0)
	{
		SSD1306_write_string("SD-Kartenfehler\n");
		SSD1306_show();
	}

	if(print_disk_info() == 0)
	{
		SSD1306_write_string("Info nicht lesbar\n");
		SSD1306_show();
	}
	else
	{
		SSD1306_show();
	}


	sd_buffer[0] = 'T';
	sd_buffer[1] = 'e';
	sd_buffer[2] = 's';
	sd_buffer[3] = 'T';
	sd_buffer[4] = 0x00;

	SSD1306_write_string("Write: ");
	SSD1306_write_string(sd_buffer);

	if(sd_raw_write(block_add, sd_buffer, read_len) == 1)
	{
		SSD1306_write_string(" O.K.\n");
	}
	else
	{
		SSD1306_write_string(" failed\n");
	}

	SSD1306_show();

	sd_buffer[0] = '-';
	sd_buffer[1] = '-';
	sd_buffer[2] = '-';
	sd_buffer[3] = '-';

	SSD1306_write_string("Clear: ");
	SSD1306_write_string(sd_buffer);
	SSD1306_write('\n');
	SSD1306_show();

	uint8_t read_ok = 0;

	if(sd_raw_read(block_add, sd_buffer, read_len) == 1)
	{
		read_ok = 1;
	}

	SSD1306_write_string("Read : ");

	for(uint16_t i = 0;i<512;i++)
	{
		char Zeichen = sd_buffer[i];
		if(Zeichen == NULL)
		break;
		SSD1306_write(Zeichen);
	}

	if(read_ok == 1)
	{
		SSD1306_write_string(" O.K.\n");
	}
	else
	{
		SSD1306_write_string(" failed\n");
	}

	SSD1306_show();

	_delay_ms(5000);

	while(1)
	{
		SSD1306_Graphic_fun();
	}

}









void Clock_init(void)
{
	OSC.XOSCCTRL = OSC_FRQRANGE_2TO9_gc | OSC_XOSCSEL_XTAL_256CLK_gc;													// Externer Quarz 8MHz, 256 Zyklen ...
	OSC.CTRL    |= OSC_XOSCEN_bm;																						// Starte mit den externen Quarz
	
	while (!(OSC.STATUS & OSC_XOSCRDY_bm));																				// Warte bis Quarz-Takt stabil
	
	OSC.PLLCTRL  = OSC_PLLSRC_XOSC_gc | 0x4;																			// PLL-Muliplexer auf 4-fach einstellen
	OSC.CTRL    |= OSC_PLLEN_bm;																						// Starte den PLL-Multiplexer
	
	while (!(OSC.STATUS & OSC_PLLRDY_bm));																				// Warte bis PLL-Takt stabil
	
	CCP          = CCP_IOREG_gc;																						// Ermglicht das Beschreiben gechtzter IO-Register
	CLK.CTRL     = CLK_SCLKSEL_PLL_gc;																					// Nutze Quarz mit 4-fach PLL-Multiplexer (32 MHz) als Takt
}


void Int_init(void)
{
	PMIC.CTRL		|= PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;											// Interrupts (Highlevel, Mediumlevel und Lowlevel freigeben)
	sei();																												// Globale Interruptfreigabe
}




uint8_t print_disk_info()
{
	struct sd_raw_info disk_info;
	
	if(!sd_raw_get_info(&disk_info))
	return 0;

	//	write_string("manuf: 0x"); uart_putc_hex(disk_info.manufacturer); write('\n');
	SSD1306_write_string("oem : "); SSD1306_write_string((char*) disk_info.oem); SSD1306_write('\n');
	SSD1306_write_string("prod: "); SSD1306_write_string((char*) disk_info.product); SSD1306_write('\n');
	//	write_string("rev:   "); uart_putc_hex(disk_info.revision); write('\n');
	//	write_string("serial:0x"); uart_putdw_hex(disk_info.serial); write('\n');
	//	write_string("date:  "); write_number(disk_info.manufacturing_month); write('/');
	//	write_number(disk_info.manufacturing_year); write('\n');
	//	write_string("size:  "); uart_putdw_dec(disk_info.capacity / 1024 / 1024); write_string("MB\n");
	//	write_string("copy:  "); write_number(disk_info.flag_copy); write('\n');
	//	write_string("wrpr:"); write_number(disk_info.flag_write_protect_temp); write('/');
	//	write_number(disk_info.flag_write_protect); write('\n');
	//	write_string("form:"); write_number(disk_info.format); write('\n');

	return 1;
}
