/*
This is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or
distribute this software, either in source code form or as a compiled
binary, for any purpose, commercial or non-commercial, and by any
means.

In jurisdictions that recognize copyright laws, the author or authors
of this software dedicate any and all copyright interest in the
software to the public domain. We make this dedication for the benefit
of the public at large and to the detriment of our heirs and
successors. We intend this dedication to be an overt act of
relinquishment in perpetuity of all present and future rights to this
software under copyright law.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

For more information, please refer to <http://unlicense.org/>
*/


#include "Common.h"


void SSD1306_drawBitmap(const uint8_t *progmem_bitmap, uint8_t height, uint8_t width, uint8_t pos_x, uint8_t pos_y) {
    uint8_t current_byte;
    uint8_t byte_width = (width + 7)/8;

    for (uint8_t current_y = 0; current_y < height; current_y++) {
        for (uint8_t current_x = 0; current_x < width; current_x++) {
            current_byte = pgm_read_byte(progmem_bitmap + current_y*byte_width + current_x/8);
            if (current_byte & (128 >> (current_x&7))) {
                SSD1306_drawPixel2(current_x+pos_x,current_y+pos_y,1);
            } else {
                SSD1306_drawPixel2(current_x+pos_x,current_y+pos_y,0);
            }
        }
    }
}

void SSD1306_drawBuffer(const uint8_t *progmem_buffer) {
    uint8_t current_byte;

    for (uint8_t y_pos = 0; y_pos < 64; y_pos++) {
        for (uint8_t x_pos = 0; x_pos < 128; x_pos++) {
            current_byte = pgm_read_byte(progmem_buffer + y_pos*16 + x_pos/8);
            if (current_byte & (128 >> (x_pos&7))) {
                SSD1306_drawPixel2(x_pos,y_pos,1);
            } else {
                SSD1306_drawPixel2(x_pos,y_pos,0);
            }
        }
    }
}

void SSD1306_drawPixel2(uint8_t pos_x, uint8_t pos_y, uint8_t pixel_status) {
    if (pos_x >= SSD1306_LCDWIDTH || pos_y >= SSD1306_LCDHEIGHT) {
        return;
    }

    if (pixel_status) {
        buffer[pos_x+(pos_y/8)*SSD1306_LCDWIDTH] |= (1 << (pos_y&7));
    } else {
        buffer[pos_x+(pos_y/8)*SSD1306_LCDWIDTH] &= ~(1 << (pos_y&7));
    }
}

void SSD1306_drawPixel1(uint8_t pos_x, uint8_t pos_y) {
    if (pos_x >= SSD1306_LCDWIDTH || pos_y >= SSD1306_LCDHEIGHT) {
        return;
    }

    buffer[pos_x+(pos_y/8)*SSD1306_LCDWIDTH] |= (1 << (pos_y&7));
}

void SSD1306_drawVLine(uint8_t x, uint8_t y, uint8_t length) {
    for (uint8_t i = 0; i < length; ++i) {
        SSD1306_drawPixel1(x,i+y);
    }
}

void SSD1306_drawHLine(uint8_t x, uint8_t y, uint8_t length) {
    for (uint8_t i = 0; i < length; ++i) {
        SSD1306_drawPixel1(i+x,y);
    }
}

void SSD1306_drawRectangle1(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2) {
    uint8_t length = x2 - x1 + 1;
    uint8_t height = y2 - y1;

    SSD1306_drawHLine(x1,y1,length);
    SSD1306_drawHLine(x1,y2,length);
    SSD1306_drawVLine(x1,y1,height);
    SSD1306_drawVLine(x2,y1,height);
}

void SSD1306_drawRectangle2(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t fill) {
    if (!fill) {
        SSD1306_drawRectangle1(x1,y1,x2,y2);
    } else {
        uint8_t length = x2 - x1 + 1;
        uint8_t height = y2 - y1;

        for (int x = 0; x < length; ++x) {
            for (int y = 0; y <= height; ++y) {
                SSD1306_drawPixel1(x1+x,y+y1);
            }
        }
    }
}

void SSD1306_clear() {
    for (uint16_t buffer_location = 0; buffer_location < SSD1306_BUFFERSIZE; buffer_location++) {
        buffer[buffer_location] = 0x00;
    }
}

void SSD1306_invert(uint8_t status) {
    SSD1306_oled_invert(status);
}

void SSD1306_show() {
    SSD1306_sendFramebuffer(buffer);
}






// Ab hier von ADAFRUIT



// draw a circle outline
void SSD1306_drawCircle(int16_t x0, int16_t y0, int16_t r, uint16_t color) 
{
	int16_t f = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x = 0;
	int16_t y = r;

	SSD1306_drawPixel2(x0, y0+r, color);
	SSD1306_drawPixel2(x0, y0-r, color);
	SSD1306_drawPixel2(x0+r, y0, color);
	SSD1306_drawPixel2(x0-r, y0, color);

	while (x<y) {
		if (f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x;
		
		SSD1306_drawPixel2(x0 + x, y0 + y, color);
		SSD1306_drawPixel2(x0 - x, y0 + y, color);
		SSD1306_drawPixel2(x0 + x, y0 - y, color);
		SSD1306_drawPixel2(x0 - x, y0 - y, color);
		SSD1306_drawPixel2(x0 + y, y0 + x, color);
		SSD1306_drawPixel2(x0 - y, y0 + x, color);
		SSD1306_drawPixel2(x0 + y, y0 - x, color);
		SSD1306_drawPixel2(x0 - y, y0 - x, color);
		
	}
}

void SSD1306_drawCircleHelper( int16_t x0, int16_t y0, int16_t r, uint8_t cornername, uint16_t color) 
{
  int16_t f     = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x     = 0;
  int16_t y     = r;

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;
    if (cornername & 0x4) {
      SSD1306_drawPixel2(x0 + x, y0 + y, color);
      SSD1306_drawPixel2(x0 + y, y0 + x, color);
    } 
    if (cornername & 0x2) {
      SSD1306_drawPixel2(x0 + x, y0 - y, color);
      SSD1306_drawPixel2(x0 + y, y0 - x, color);
    }
    if (cornername & 0x8) {
      SSD1306_drawPixel2(x0 - y, y0 + x, color);
      SSD1306_drawPixel2(x0 - x, y0 + y, color);
    }
    if (cornername & 0x1) {
      SSD1306_drawPixel2(x0 - y, y0 - x, color);
      SSD1306_drawPixel2(x0 - x, y0 - y, color);
    }
  }
}

void SSD1306_fillCircle(int16_t x0, int16_t y0, int16_t r, uint16_t color) 
{
  SSD1306_drawFastVLine(x0, y0-r, 2*r+1, color);
  SSD1306_fillCircleHelper(x0, y0, r, 3, 0, color);
}

// used to do circles and roundrects!
void SSD1306_fillCircleHelper(int16_t x0, int16_t y0, int16_t r, uint8_t cornername, int16_t delta, uint16_t color) 
{

  int16_t f     = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x     = 0;
  int16_t y     = r;

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;

    if (cornername & 0x1) {
      SSD1306_drawFastVLine(x0+x, y0-y, 2*y+1+delta, color);
      SSD1306_drawFastVLine(x0+y, y0-x, 2*x+1+delta, color);
    }
    if (cornername & 0x2) {
      SSD1306_drawFastVLine(x0-x, y0-y, 2*y+1+delta, color);
      SSD1306_drawFastVLine(x0-y, y0-x, 2*x+1+delta, color);
    }
  }
}

// bresenham's algorithm - thx wikpedia
void SSD1306_drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t color) 
{
  int16_t steep = abs(y1 - y0) > abs(x1 - x0);
  if (steep) {
    swap(x0, y0);
    swap(x1, y1);
  }

  if (x0 > x1) {
    swap(x0, x1);
    swap(y0, y1);
  }

  int16_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);

  int16_t err = dx / 2;
  int16_t ystep;

  if (y0 < y1) {
    ystep = 1;
  } else {
    ystep = -1;
  }

  for (; x0<=x1; x0++) {
    if (steep) {
      SSD1306_drawPixel2(y0, x0, color);
    } else {
      SSD1306_drawPixel2(x0, y0, color);
    }
    err -= dy;
    if (err < 0) {
      y0 += ystep;
      err += dx;
    }
  }
}


// draw a rectangle
void SSD1306_drawRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color) 
{
  SSD1306_drawFastHLine(x, y, w, color);
  SSD1306_drawFastHLine(x, y+h-1, w, color);
  SSD1306_drawFastVLine(x, y, h, color);
  SSD1306_drawFastVLine(x+w-1, y, h, color);
}

void SSD1306_drawFastVLine(int16_t x, int16_t y, int16_t h, uint16_t color) 
{
  // stupidest version - update in subclasses if desired!
  SSD1306_drawLine(x, y, x, y+h-1, color);
}


void SSD1306_drawFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color) 
{
  // stupidest version - update in subclasses if desired!
  SSD1306_drawLine(x, y, x+w-1, y, color);
}

void SSD1306_fillRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color) 
{
  // stupidest version - update in subclasses if desired!
  for (int16_t i=x; i<x+w; i++) {
    SSD1306_drawFastVLine(i, y, h, color); 
  }
}


void SSD1306_fillScreen(uint16_t color) 
{
  SSD1306_fillRect(0, 0, _width, _height, color);
}

// draw a_
void SSD1306_drawRoundRect(int16_t x, int16_t y, int16_t w, int16_t h, int16_t r, uint16_t color) 
{
  // smarter version
  SSD1306_drawFastHLine(x+r  , y    , w-2*r, color); // Top
  SSD1306_drawFastHLine(x+r  , y+h-1, w-2*r, color); // Bottom
  SSD1306_drawFastVLine(  x    , y+r  , h-2*r, color); // Left
  SSD1306_drawFastVLine(  x+w-1, y+r  , h-2*r, color); // Right
  // draw four corners
  SSD1306_drawCircleHelper(x+r    , y+r    , r, 1, color);
  SSD1306_drawCircleHelper(x+w-r-1, y+r    , r, 2, color);
  SSD1306_drawCircleHelper(x+w-r-1, y+h-r-1, r, 4, color);
  SSD1306_drawCircleHelper(x+r    , y+h-r-1, r, 8, color);
}

// fill a rounded rectangle!
void SSD1306_fillRoundRect(int16_t x, int16_t y, int16_t w, int16_t h, int16_t r, uint16_t color) 
{
  // smarter version
  SSD1306_fillRect(x+r, y, w-2*r, h, color);

  // draw four corners
  SSD1306_fillCircleHelper(x+w-r-1, y+r, r, 1, h-2*r-1, color);
  SSD1306_fillCircleHelper(x+r    , y+r, r, 2, h-2*r-1, color);
}

// draw a triangle!
void SSD1306_drawTriangle(int16_t x0, int16_t y0, int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint16_t color) 
{
  SSD1306_drawLine(x0, y0, x1, y1, color);
  SSD1306_drawLine(x1, y1, x2, y2, color);
  SSD1306_drawLine(x2, y2, x0, y0, color);
}

// fill a triangle!
void SSD1306_fillTriangle ( int16_t x0, int16_t y0, int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint16_t color) 
{

  int16_t a, b, y, last;

  // Sort coordinates by Y order (y2 >= y1 >= y0)
  if (y0 > y1) {
    swap(y0, y1); swap(x0, x1);
  }
  if (y1 > y2) {
    swap(y2, y1); swap(x2, x1);
  }
  if (y0 > y1) {
    swap(y0, y1); swap(x0, x1);
  }

  if(y0 == y2) { // Handle awkward all-on-same-line case as its own thing
    a = b = x0;
    if(x1 < a)      a = x1;
    else if(x1 > b) b = x1;
    if(x2 < a)      a = x2;
    else if(x2 > b) b = x2;
    SSD1306_drawFastHLine(a, y0, b-a+1, color);
    return;
  }

  int16_t
    dx01 = x1 - x0,
    dy01 = y1 - y0,
    dx02 = x2 - x0,
    dy02 = y2 - y0,
    dx12 = x2 - x1,
    dy12 = y2 - y1,
    sa   = 0,
    sb   = 0;

  // For upper part of triangle, find scanline crossings for segments
  // 0-1 and 0-2.  If y1=y2 (flat-bottomed triangle), the scanline y1
  // is included here (and second loop will be skipped, avoiding a /0
  // error there), otherwise scanline y1 is skipped here and handled
  // in the second loop...which also avoids a /0 error here if y0=y1
  // (flat-topped triangle).
  if(y1 == y2) last = y1;   // Include y1 scanline
  else         last = y1-1; // Skip it

  for(y=y0; y<=last; y++) {
    a   = x0 + sa / dy01;
    b   = x0 + sb / dy02;
    sa += dx01;
    sb += dx02;
    /* longhand:
    a = x0 + (x1 - x0) * (y - y0) / (y1 - y0);
    b = x0 + (x2 - x0) * (y - y0) / (y2 - y0);
    */
    if(a > b) swap(a,b);
    SSD1306_drawFastHLine(a, y, b-a+1, color);
  }

  // For lower part of triangle, find scanline crossings for segments
  // 0-2 and 1-2.  This loop is skipped if y1=y2.
  sa = dx12 * (y - y1);
  sb = dx02 * (y - y0);
  for(; y<=y2; y++) {
    a   = x1 + sa / dy12;
    b   = x0 + sb / dy02;
    sa += dx12;
    sb += dx02;
    /* longhand:
    a = x1 + (x2 - x1) * (y - y1) / (y2 - y1);
    b = x0 + (x2 - x0) * (y - y0) / (y2 - y0);
    */
    if(a > b) swap(a,b);
    SSD1306_drawFastHLine(a, y, b-a+1, color);
  }
}


/*
void drawBitmap(int16_t x, int16_t y, 
			      const uint8_t *bitmap, int16_t w, int16_t h,
			      uint16_t color) {
  for (int16_t j=0; j<h; j++) {
    for (int16_t i=0; i<w; i++ ) {
      if (pgm_read_byte(bitmap + i + (j/8)*w) & _BV(j%8)) {
	drawPixel2(x+i, y+j, color);
      }
    }
  }
}
*/


void SSD1306_write_string(const uint8_t *ptr)
{
	while(*ptr) 
		SSD1306_write(*ptr++);
}


void SSD1306_write(uint8_t c) 
{
  if (c == '\n') {
    cursor_y += textsize*8;
    cursor_x = 0;
  } else if (c == '\r') {
    // skip em
  } else {
    SSD1306_drawChar(cursor_x, cursor_y, c, textcolor, textbgcolor, textsize);
    cursor_x += textsize*6;
    if (wrap && (cursor_x > (_width - textsize*6))) {
      cursor_y += textsize*8;
      cursor_x = 0;
    }
  }
}

// draw a character
void SSD1306_drawChar(int16_t x, int16_t y, unsigned char c, uint16_t color, uint16_t bg, uint8_t size) 
{

  if((x >= _width)            || // Clip right
     (y >= _height)           || // Clip bottom
     ((x + 5 * size - 1) < 0) || // Clip left
     ((y + 8 * size - 1) < 0))   // Clip top
    return;

  for (int8_t i=0; i<6; i++ ) {
    uint8_t line;
    if (i == 5) 
      line = 0x0;
    else 
      line = pgm_read_byte(font+(c*5)+i);
    for (int8_t j = 0; j<8; j++) {
      if (line & 0x1) {
        if (size == 1) // default size
          SSD1306_drawPixel2(x+i, y+j, color);
        else {  // big size
          SSD1306_fillRect(x+(i*size), y+(j*size), size, size, color);
        } 
      } else if (bg != color) {
        if (size == 1) // default size
          SSD1306_drawPixel2(x+i, y+j, bg);
        else {  // big size
          SSD1306_fillRect(x+i*size, y+j*size, size, size, bg);
        } 	
      }
      line >>= 1;
    }
  }
}


// Print Number
void SSD1306_write_number(int16_t Data)
{
	uint8_t D1=0,D2=0,D3=0,D4=0,D5=0;
	if(Data<0)
	{
		Data=-Data;
		SSD1306_write('-');
	}
	else SSD1306_write(' ');
	
	while (Data>=10000)	{ D5++; Data-=10000; }
	while (Data>=1000)	{ D4++; Data-=1000; }
	while (Data>=100)	{ D3++; Data-=100; }
	while (Data>=10)	{ D2++; Data-=10; }
	while (Data>=1)		{ D1++; Data-=1; }
	SSD1306_write(0x30+D5);
	SSD1306_write(0x30+D4);
	SSD1306_write(0x30+D3);
	SSD1306_write(0x30+D2);
	SSD1306_write(0x30+D1);
	SSD1306_write(' ');
}



void SSD1306_setCursor(int16_t x, int16_t y) 
{
  cursor_x = x;
  cursor_y = y;
}


void SSD1306_setTextSize(uint8_t s) 
{
  textsize = (s > 0) ? s : 1;
}


void SSD1306_setTextColor1(uint16_t c) 
{
  textcolor = c;
  textbgcolor = c; 
  // for 'transparent' background, we'll set the bg 
  // to the same as fg instead of using a flag
}

 void SSD1306_setTextColor2(uint16_t c, uint16_t b) 
 {
   textcolor = c;
   textbgcolor = b; 
 }

void SSD1306_setTextWrap(char w) 
{
  wrap = w;
}

uint8_t SSD1306_getRotation(void) 
{
  rotation %= 4;
  return rotation;
}

void SSD1306_setRotation(uint8_t x) 
{
  x %= 4;  // cant be higher than 3
  rotation = x;
  switch (x) {
  case 0:
  case 2:
    _width = WIDTH;
    _height = HEIGHT;
    break;
  case 1:
  case 3:
    _width = HEIGHT;
    _height = WIDTH;
    break;
  }
}




// return the size of the display which depends on the rotation!
int16_t SSD1306_width(void) {
	return _width;
}

int16_t SSD1306_height(void) {
	return _height;
}
  