/*
This is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or
distribute this software, either in source code form or as a compiled
binary, for any purpose, commercial or non-commercial, and by any
means.

In jurisdictions that recognize copyright laws, the author or authors
of this software dedicate any and all copyright interest in the
software to the public domain. We make this dedication for the benefit
of the public at large and to the detriment of our heirs and
successors. We intend this dedication to be an overt act of
relinquishment in perpetuity of all present and future rights to this
software under copyright law.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

For more information, please refer to <http://unlicense.org/>
*/

#ifndef _FRAMEBUFFER_H_
#define _FRAMEBUFFER_H_

#include "Common.h"


#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include <util/delay.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "SSD1306.h"


void SSD1306_drawBitmap(const uint8_t *bitmap, uint8_t height, uint8_t width, uint8_t pos_x, uint8_t pos_y);
void SSD1306_drawBuffer(const uint8_t *buffer);
void SSD1306_drawPixel2(uint8_t pos_x, uint8_t pos_y, uint8_t pixel_status);
void SSD1306_drawPixel1(uint8_t pos_x, uint8_t pos_y);
void SSD1306_drawVLine(uint8_t x, uint8_t y, uint8_t length);
void SSD1306_drawHLine(uint8_t pos_y, uint8_t start, uint8_t length);
void SSD1306_drawRectangle1(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void SSD1306_drawRectangle2(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t fill);
void SSD1306_invert(uint8_t status);
void SSD1306_clear();
void SSD1306_show();

uint8_t buffer[1024];





#define swap(a, b) { int16_t t = a; a = b; b = t; }
	


// these are 'generic' drawing functions, so we can share them!
void SSD1306_drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t color);
void SSD1306_drawFastVLine(int16_t x, int16_t y, int16_t h, uint16_t color);
void SSD1306_drawFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color);
void SSD1306_drawRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color);
void SSD1306_fillRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color);
void SSD1306_fillScreen(uint16_t color);

void SSD1306_drawCircle(int16_t x0, int16_t y0, int16_t r, uint16_t color);
void SSD1306_drawCircleHelper(int16_t x0, int16_t y0, int16_t r, uint8_t cornername, uint16_t color);
void SSD1306_fillCircle(int16_t x0, int16_t y0, int16_t r, uint16_t color);
void SSD1306_fillCircleHelper(int16_t x0, int16_t y0, int16_t r, uint8_t cornername, int16_t delta, uint16_t color);

void SSD1306_drawTriangle(int16_t x0, int16_t y0, int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint16_t color);
void SSD1306_fillTriangle(int16_t x0, int16_t y0, int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint16_t color);
void SSD1306_drawRoundRect(int16_t x0, int16_t y0, int16_t w, int16_t h, int16_t radius, uint16_t color);
void SSD1306_fillRoundRect(int16_t x0, int16_t y0, int16_t w, int16_t h, int16_t radius, uint16_t color);

//void drawBitmap(int16_t x, int16_t y, const uint8_t *bitmap, int16_t w, int16_t h, uint16_t color);
void SSD1306_drawChar(int16_t x, int16_t y, unsigned char c, uint16_t color, uint16_t bg, uint8_t size);
void SSD1306_write_string(const uint8_t *ptr);
void SSD1306_write(uint8_t);

void SSD1306_setCursor(int16_t x, int16_t y);
void SSD1306_setTextColor1(uint16_t c);
void SSD1306_setTextColor2(uint16_t c, uint16_t bg);
void SSD1306_setTextSize(uint8_t s);
void SSD1306_setTextWrap(char w);

int16_t SSD1306_height(void);
int16_t SSD1306_width(void);

void SSD1306_setRotation(uint8_t r);
uint8_t SSD1306_getRotation(void);

int16_t  WIDTH, HEIGHT;   // this is the 'raw' display w/h - never changes
int16_t  _width, _height; // dependent on rotation
int16_t  cursor_x, cursor_y;
uint16_t textcolor, textbgcolor;
uint8_t  textsize;
uint8_t  rotation;
char     wrap; // If set, 'wrap' text at right edge of display

#endif