/*
 * SSD1289.c
 *
 * Created: 23.06.2015 08:53:01
 *  Author: Rolf Degen
 */ 


#include <avr/io.h>
#include <util/delay.h>
#include "SSD1289.h"
#include <math.h>

//									 CHAR   		       		   ASCII-DEC
const int FONT[95][5] ={ 
	
	{0x00, 0x00, 0x00, 0x00, 0x00}, // SPACE                 		   32
	{0x00, 0x00, 0x5F, 0x00, 0x00}, // !                             33
	{0x00, 0x03, 0x00, 0x03, 0x00}, // "                             34
	{0x14, 0x3E, 0x14, 0x3E, 0x14}, // #                             35
	{0x24, 0x2A, 0x7F, 0x2A, 0x12}, // $                             36
	{0x43, 0x33, 0x08, 0x66, 0x61}, // %                             37
	{0x36, 0x49, 0x55, 0x22, 0x50}, // &                             38
	{0x00, 0x05, 0x03, 0x00, 0x00}, // '                             39
	{0x00, 0x1C, 0x22, 0x41, 0x00}, // (                             40
	{0x00, 0x41, 0x22, 0x1C, 0x00}, // )                             41
	{0x14, 0x08, 0x3E, 0x08, 0x14}, // *                             42
	{0x08, 0x08, 0x3E, 0x08, 0x08}, // +                             43
	{0x00, 0x50, 0x30, 0x00, 0x00}, // ,                             44
	{0x08, 0x08, 0x08, 0x08, 0x08}, // -                             45
	{0x00, 0x60, 0x60, 0x00, 0x00}, // .                             46
	{0x20, 0x10, 0x08, 0x04, 0x02}, // /                             47
	{0x3E, 0x51, 0x49, 0x45, 0x3E}, // 0                             48
	{0x00, 0x04, 0x02, 0x7F, 0x00}, // 1                             49
	{0x42, 0x61, 0x51, 0x49, 0x46}, // 2                             50
	{0x22, 0x41, 0x49, 0x49, 0x36}, // 3                             51
	{0x18, 0x14, 0x12, 0x7F, 0x10}, // 4                             52
	{0x27, 0x45, 0x45, 0x45, 0x39}, // 5                             53
	{0x3E, 0x49, 0x49, 0x49, 0x32}, // 6                             54
	{0x01, 0x01, 0x71, 0x09, 0x07}, // 7                             55
	{0x36, 0x49, 0x49, 0x49, 0x36}, // 8                             56
	{0x26, 0x49, 0x49, 0x49, 0x3E}, // 9                             57
	{0x00, 0x36, 0x36, 0x00, 0x00}, // :                             58
	{0x00, 0x56, 0x36, 0x00, 0x00}, // ;                             59
	{0x08, 0x14, 0x22, 0x41, 0x00}, // <                             60
	{0x14, 0x14, 0x14, 0x14, 0x14}, // =                             61
	{0x00, 0x41, 0x22, 0x14, 0x08}, // >                             62
	{0x02, 0x01, 0x51, 0x09, 0x06}, // ?                             63
	{0x3E, 0x41, 0x59, 0x55, 0x5E}, // @                             64
	{0x7E, 0x09, 0x09, 0x09, 0x7E}, // A                             65
	{0x7F, 0x49, 0x49, 0x49, 0x36}, // B                             66
	{0x3E, 0x41, 0x41, 0x41, 0x22}, // C                             67
	{0x7F, 0x41, 0x41, 0x41, 0x3E}, // D                             68
	{0x7F, 0x49, 0x49, 0x49, 0x41}, // E                             69
	{0x7F, 0x09, 0x09, 0x09, 0x01}, // F                             70
	{0x3E, 0x41, 0x41, 0x49, 0x3A}, // G                             71
	{0x7F, 0x08, 0x08, 0x08, 0x7F}, // H                             72
	{0x00, 0x41, 0x7F, 0x41, 0x00}, // I                             73
	{0x30, 0x40, 0x40, 0x40, 0x3F}, // J                             74
	{0x7F, 0x08, 0x14, 0x22, 0x41}, // K                             75
	{0x7F, 0x40, 0x40, 0x40, 0x40}, // L                             76
	{0x7F, 0x02, 0x0C, 0x02, 0x7F}, // M                             77
	{0x7F, 0x02, 0x04, 0x08, 0x7F}, // N                             78
	{0x3E, 0x41, 0x41, 0x41, 0x3E}, // O                             79
	{0x7F, 0x09, 0x09, 0x09, 0x06}, // P                             80
	{0x1E, 0x21, 0x21, 0x21, 0x5E}, // Q                             81
	{0x7F, 0x09, 0x09, 0x09, 0x76}, // R                             82
	{0x26, 0x49, 0x49, 0x49, 0x32}, // S                             83
	{0x01, 0x01, 0x7F, 0x01, 0x01}, // T                             84
	{0x3F, 0x40, 0x40, 0x40, 0x3F}, // U                             85
	{0x1F, 0x20, 0x40, 0x20, 0x1F}, // V                             86
	{0x7F, 0x20, 0x10, 0x20, 0x7F}, // W                             87
	{0x41, 0x22, 0x1C, 0x22, 0x41}, // X                             88
	{0x07, 0x08, 0x70, 0x08, 0x07}, // Y                             89
	{0x61, 0x51, 0x49, 0x45, 0x43}, // Z                             90
	{0x00, 0x7F, 0x41, 0x00, 0x00}, // [                             91
	{0x02, 0x04, 0x08, 0x10, 0x20}, // \                             92
	{0x00, 0x00, 0x41, 0x7F, 0x00}, // ]                             93
	{0x04, 0x02, 0x01, 0x02, 0x04}, // ^                             94
	{0x40, 0x40, 0x40, 0x40, 0x40}, // _                             95
	{0x00, 0x01, 0x02, 0x04, 0x00}, // `                             96
	{0x20, 0x54, 0x54, 0x54, 0x78}, // a                             97
	{0x7F, 0x44, 0x44, 0x44, 0x38}, // b                             98
	{0x38, 0x44, 0x44, 0x44, 0x44}, // c                             99
	{0x38, 0x44, 0x44, 0x44, 0x7F}, // d                             100
	{0x38, 0x54, 0x54, 0x54, 0x18}, // e                             101
	{0x04, 0x04, 0x7E, 0x05, 0x05}, // f                             102
	{0x08, 0x54, 0x54, 0x54, 0x3C}, // g                             103
	{0x7F, 0x08, 0x04, 0x04, 0x78}, // h                             104
	{0x00, 0x44, 0x7D, 0x40, 0x00}, // i                             105
	{0x20, 0x40, 0x44, 0x3D, 0x00}, // j                             106
	{0x7F, 0x10, 0x28, 0x44, 0x00}, // k                             107
	{0x00, 0x41, 0x7F, 0x40, 0x00}, // l                             108
	{0x7C, 0x04, 0x78, 0x04, 0x78}, // m                             109
	{0x7C, 0x08, 0x04, 0x04, 0x78}, // n                             110
	{0x38, 0x44, 0x44, 0x44, 0x38}, // o                             111
	{0x7C, 0x14, 0x14, 0x14, 0x08}, // p                             112
	{0x08, 0x14, 0x14, 0x14, 0x7C}, // q                             113
	{0x00, 0x7C, 0x08, 0x04, 0x04}, // r                             114
	{0x48, 0x54, 0x54, 0x54, 0x20}, // s                             115
	{0x04, 0x04, 0x3F, 0x44, 0x44}, // t                             116
	{0x3C, 0x40, 0x40, 0x20, 0x7C}, // u                             117
	{0x1C, 0x20, 0x40, 0x20, 0x1C}, // v                             118
	{0x3C, 0x40, 0x30, 0x40, 0x3C}, // w                             119
	{0x44, 0x28, 0x10, 0x28, 0x44}, // x                             120
	{0x0C, 0x50, 0x50, 0x50, 0x3C}, // y                             121
	{0x44, 0x64, 0x54, 0x4C, 0x44}, // z                             122
	{0x00, 0x08, 0x36, 0x41, 0x41}, // {                             123
	{0x00, 0x00, 0x7F, 0x00, 0x00}, // |                             124
	{0x41, 0x41, 0x36, 0x08, 0x00}, // }                             125
	{0x02, 0x01, 0x02, 0x04, 0x02}};// ~                             126



uint8_t SPIDir = 0 ;

void spiopen(void)
{
	SPIPort.DIRCLR	= (1<<SPIMI);
	SPIPort.DIRSET	= (1<<SPIMO) | (1<<SPISC) | (1<<SPISS) ; // Make sure slave select is an output
	SPID.CTRL		= SPI_CLK2X_bm | SPI_ENABLE_bm | SPI_MASTER_bm | SPI_MODE_0_gc | SPI_PRESCALER_DIV4_gc;
}


void spiclose(void)
{
	SPID.CTRL		= 0;
}


uint8_t spi(uint8_t val)
{
	SPID.DATA		= val ;
	while(!(SPID.STATUS & (1<<7)));
	_delay_ms(10);
	return SPID.DATA;
}




void SSD1289_init(void)
{
	SET_RESET;
	_delay_ms(5);
	CLR_RESET;
	_delay_ms(15);
	SET_RESET;
	_delay_ms(15);
	CLR_CS;  //enable cs
	SSD1289_main_W_com_data(0x00,0x0001);
	SSD1289_main_W_com_data(0x03,0xA8A4);
	SSD1289_main_W_com_data(0x0C,0x0000);
	SSD1289_main_W_com_data(0x0D,0x080C);
	SSD1289_main_W_com_data(0x0E,0x2B00);
	SSD1289_main_W_com_data(0x1E,0x00B7);
	SSD1289_main_W_com_data(0x01,0x2B3F);
	SSD1289_main_W_com_data(0x02,0x0600);
	SSD1289_main_W_com_data(0x10,0x0000);
	SSD1289_main_W_com_data(0x11,0x6070);
	SSD1289_main_W_com_data(0x05,0x0000);
	SSD1289_main_W_com_data(0x06,0x0000);
	SSD1289_main_W_com_data(0x16,0xEF1C);
	SSD1289_main_W_com_data(0x17,0x0003);
	SSD1289_main_W_com_data(0x07,0x0233);
	SSD1289_main_W_com_data(0x0B,0x0000);
	SSD1289_main_W_com_data(0x0F,0x0000);
	SSD1289_main_W_com_data(0x41,0x0000);
	SSD1289_main_W_com_data(0x42,0x0000);
	SSD1289_main_W_com_data(0x48,0x0000);
	SSD1289_main_W_com_data(0x49,0x013F);
	SSD1289_main_W_com_data(0x4A,0x0000);
	SSD1289_main_W_com_data(0x4B,0x0000);
	SSD1289_main_W_com_data(0x44,0xEF00);
	SSD1289_main_W_com_data(0x45,0x0000);
	SSD1289_main_W_com_data(0x46,0x013F);
	SSD1289_main_W_com_data(0x30,0x0707);
	SSD1289_main_W_com_data(0x31,0x0204);
	SSD1289_main_W_com_data(0x32,0x0204);
	SSD1289_main_W_com_data(0x33,0x0502);
	SSD1289_main_W_com_data(0x34,0x0507);
	SSD1289_main_W_com_data(0x35,0x0204);
	SSD1289_main_W_com_data(0x36,0x0204);
	SSD1289_main_W_com_data(0x37,0x0502);
	SSD1289_main_W_com_data(0x3A,0x0302);
	SSD1289_main_W_com_data(0x3B,0x0302);
	SSD1289_main_W_com_data(0x23,0x0000);
	SSD1289_main_W_com_data(0x24,0x0000);
	SSD1289_main_W_com_data(0x25,0x8000);
	SSD1289_main_W_com_data(0x4f,0x0000);
	SSD1289_main_W_com_data(0x4e,0x0000);
	SSD1289_LCD_Write_COM(0x00,0x22);
	SET_CS; //disable  cs
}
void SSD1289_Address_set(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2)
{
	SSD1289_LCD_Write_COM(0x00,0x20);SSD1289_LCD_Write_DATA(x1>>8,x1);
	SSD1289_LCD_Write_COM(0x00,0x21);SSD1289_LCD_Write_DATA(y1>>8,y1);
	SSD1289_LCD_Write_COM(0x00,0x50);SSD1289_LCD_Write_DATA(x1>>8,x1);
	SSD1289_LCD_Write_COM(0x00,0x52);SSD1289_LCD_Write_DATA(y1>>8,y1);
	SSD1289_LCD_Write_COM(0x00,0x51);SSD1289_LCD_Write_DATA(x2>>8,x2);
	SSD1289_LCD_Write_COM(0x00,0x53);SSD1289_LCD_Write_DATA(y2>>8,y2);
	SSD1289_LCD_Write_COM(0x00,0x22);
}
void SSD1289_set_Cursor(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2)
{
	SSD1289_main_W_com_data(0x44,(x2<<8)+x1);
	SSD1289_main_W_com_data(0x45,y1);
	SSD1289_main_W_com_data(0x46,y2);
	SSD1289_main_W_com_data(0x4E,x1);
	SSD1289_main_W_com_data(0x4F,y1);
	SSD1289_LCD_Write_COM(0x00,0x22);
}
void SSD1289_main_W_com_data(int com1,int dat1)
{
	SSD1289_LCD_Write_COM(com1>>8,com1);
	SSD1289_LCD_Write_DATA(dat1>>8,dat1);
}
void SSD1289_LCD_Write_COM(char VH,char VL)      //snet command
{
	CLR_RS;
	SSD1289_LCD_Writ_Bus(VH,VL);
}
void SSD1289_LCD_Write_DATA(char VH,char VL) //sent data
{
	SET_RS;
	SSD1289_LCD_Writ_Bus(VH,VL);
}
void SSD1289_LCD_Writ_Bus(char VH,char VL)   //Write Data
{
	uint8_t retval;
//	LCD_DataPortH=VH;
	retval = spi(VH);
	
//	LCD_DataPortL=VL;
	retval = spi(VL);

	CLR_74HC595;
	SET_74HC595;							//Daten raus
	
//	_delay_ms(20);
	
	CLR_WR;
	SET_WR;
}
void SSD1289_Paint(char VH, char VL)
{
	long i;
	CLR_CS;
	SSD1289_set_Cursor(0,0,239,320);
	for(i=0;i<76800;i++)
	{
		SSD1289_LCD_Write_DATA(VH,VL);
	}
	SET_CS;
}
void SSD1289_fillrectangle(int x1, int y1, int x2, int y2, char VH, char VL)
{
	CLR_CS;
	SSD1289_set_Cursor(x1,y1,x2-1,y2);
	for(long i = 0; i < ((x2-x1)*(y2-y1)); i++)
	{
		SSD1289_LCD_Write_DATA(VH, VL);
	}
	SET_CS;
}
void SSD1289_vline(int x, int y, int l, char VH, char VL)
{
	int i;
	CLR_CS;
	SSD1289_set_Cursor(x,y,x,y+l);
	for(i=0;i<l;i++)
	{
		SSD1289_LCD_Write_DATA(VH,VL);
	}
	SET_CS;
}
void SSD1289_hline(int x, int y, int l, char VH, char VL)
{
	int i;
	CLR_CS;
	SSD1289_set_Cursor(x,y,x+l,y);
	for(i=0;i<l+1;i++)
	{
		SSD1289_LCD_Write_DATA(VH,VL);
	}
	SET_CS;
}
void SSD1289_pixel(int x, int y, char VH, char VL)
{
	CLR_CS;
	SSD1289_set_Cursor(x,y,x,y);
	SSD1289_LCD_Write_DATA(VH,VL);
	SET_CS;
}
void SSD1289_drawallchars()
{
	int tempx = 0;
	int tempy = 10;
	for(int h = 93; h >= 0; h--)
	{
		for(int i = 0; i < 5; i++)
		{
			for(int j = 0; j < 8; j++)
			{
				if(FONT[h][4-i] & 1<<j)
				{
					SSD1289_pixel(j + tempy, i + tempx, BLUE);
				}
			}
		}
		tempx += 6;
		if(tempx > 310)
		{
			tempy += 10;
			tempx= 0;
		}
	}
}
void SSD1289_drawchar(int ascii, int x, int y, int multiplier, char VH, char VL)
{
	int posx = 0;
	int posy = 0;
//	int offsetx = 0;
//	int offsety = 0;
	for(int i = 0; i < 5; i++)
	{
		for(int j = 0; j < 8; j++)
		{
			if(FONT[ascii][4-i] & 1<<j)
			{
				for(posy = 0;posy < multiplier; posy++)
				{
					for(posx = 0;posx < multiplier; posx++)
					{
						SSD1289_pixel(j*multiplier + y + posy, i*multiplier + x + posx, VH, VL);
					}
				}
			}
		}
	}
}
void SSD1289_drawstring(char str[], int x, int y, int multiplier, char VH, char VL)
{
	for(int counter = 0; counter < SSD1289_strlen(str); counter++)
	{
		SSD1289_drawchar(str[counter]-32, x, y, multiplier, VH, VL);
		x -= 6*multiplier;
	}
}
int SSD1289_strlen(const char *str)
{
	const char *s;
	for (s = str; *s; ++s)
	;
	return (s - str);
}
void SSD1289_rectangle(int x, int y, int height, int width, char VH, char VL)
{
	SSD1289_hline(x,y,width, VH, VL);
	SSD1289_vline(x+width,y, height, VH, VL);
	SSD1289_hline(x,y + height, width, VH, VL);
	SSD1289_vline(x,y,height, VH, VL);
}
void SSD1289_drawrighttriangle(int x, int y, int height)
{
	int width = (height/2)+1;
	for(int i = 0; i < width; i++)
	{
		SSD1289_hline(x, y, height, WHITE);
		x++;
		y--;
		height -= 2;
	}
}
void SSD1289_drawlefttriangle(int x, int y, int height)
{
	int width = (height/2)+1;
	for(int i = 0; i < width; i++)
	{
		SSD1289_hline(x, y, height, WHITE);
		x++;
		y++;
		height -= 2;
	}
}